import torch 
import time
from Zuba.Model_training import LanguageClassifier
from Zuba.DataPreprocessing.process import Return
Return=Return() 
train_loader,test_loader,val_loader=Return.Return()
import torch.nn as nn
device= torch.device("cuda" if torch.cuda.is_available() else "cpu")

def cal_accuracy(data_loader,model,device,num_batches=None):
    model.eval()
    correct_pred,num_examples=0,0
    if num_batches is None:
        num_batches=len(data_loader)
    else:
        num_batches=min(num_batches,len(data_loader))
    for i,(input_ids,labels) in enumerate(data_loader):
     if i<num_batches:
        input_ids=input_ids.to(device)
        labels=labels.to(device)
        with torch.no_grad():
          probas=torch.softmax(model(input_ids),dim=1)
        label_preds=torch.argmax(probas,dim=1)
        correct_pred+=(label_preds==labels).sum().item()
        num_examples+=labels.size(0)
     else:
        break
    return correct_pred/num_examples
       
    
class TrainClassifier:
    def __init__(self, model, train_loader, val_loader, test_loader, device, learning_rate=3e-5, epochs=10):
        self.model = model
        self.train_loader = train_loader
        self.val_loader = val_loader
        self.test_loader = test_loader
        self.device = device
        self.learning_rate = learning_rate
        self.epochs = epochs

        self.train_loss, self.val_loss, self.test_loss = [], [], []
        self.train_acc, self.val_acc, self.test_acc = [], [], []
        self.epochs_list = []

        self.criterion = nn.CrossEntropyLoss()
        self.optimizer = torch.optim.AdamW(self.model.parameters(), lr=self.learning_rate)

    def train(self):
        for epoch in range(self.epochs):
            self.model.train()
            epoch_loss = 0

            for input_ids, labels in self.train_loader:
                input_ids = input_ids.to(self.device)
                labels = labels.to(self.device)

                self.optimizer.zero_grad()
                outputs = self.model(input_ids)  # add attention_mask if needed
                loss = self.criterion(outputs, labels)
                loss.backward()
                self.optimizer.step()

                epoch_loss += loss.item()

            epoch_loss /= len(self.train_loader)
            self.train_loss.append(epoch_loss)
            print(f"Epoch {epoch+1}/{self.epochs}, Loss={epoch_loss:.4f}")

            # Evaluate
            self.model.eval()
            with torch.no_grad():
                train_acc = cal_accuracy(self.train_loader, self.model, self.device, num_batches=10)
                val_acc = cal_accuracy(self.val_loader, self.model, self.device, num_batches=10)
                test_acc = cal_accuracy(self.test_loader, self.model, self.device, num_batches=10)

            self.train_acc.append(train_acc)
            self.val_acc.append(val_acc)
            self.test_acc.append(test_acc)
            self.epochs_list.append(epoch + 1)

            print(f"Epoch {epoch+1}/{self.epochs}, "
                  f"Train Acc: {train_acc*100:.2f}%, "
                  f"Val Acc: {val_acc*100:.2f}%, "
                  f"Test Acc: {test_acc*100:.2f}%")
        torch.save(self.model.state_dict(), "language_classifier_model.pth")
        return self.train_acc, self.val_acc, self.test_acc, self.epochs_list

# initiate training
if __name__=="__main__":
  start_time=time.time()
  Train=TrainClassifier(model=LanguageClassifier(num_labels=4).to(device),train_loader=train_loader,val_loader=val_loader,test_loader=test_loader,device=device,learning_rate=3e-5,epochs=3)
  train_acc,val_acc,test_acc,epoch_list=Train.train()
  end_time = time.time()
  execution_time=(end_time - start_time)/60
  print(f"Training time: {execution_time} minutes")



#this class will create dataloaders for our datasets
import tiktoken
from Zuba.DataPreprocessing.process import Tokenizer
import torch
from torch.utils.data import DataLoader
torch.manual_seed(42)
class DataLoader:
    def __init__(self,batch_size,num_workers,dataset=None,drop_last=True,shuffle=True):
      self.dataset=dataset
      self.batch_size=batch_size
      self.num_workers=num_workers
      self.drop_last=drop_last
    def get_train_loader(self,dataset):
        self.dataset=dataset
        train_loader=DataLoader(
            dataset=self.dataset,
            batch_size=self.batch_size,
            num_workers=self.num_workers,
            drop_last=self.drop_last,
            shuffle=self.shuffle
        )
        return train_loader
    def get_val_loader(self,dataset):
        self.dataset=dataset
        val_loader=DataLoader(
            dataset=self.dataset,
            batch_size=self.batch_size,
            num_workers=self.num_workers,
            drop_last=self.drop_last,
            shuffle=self.shuffle
        )
        return val_loader
    def get_test_loader(self,dataset):
        self.dataset=dataset
        test_loader=DataLoader(
            dataset=self.dataset,
            batch_size=self.batch_size,
            num_workers=self.num_workers,
            drop_last=self.drop_last,
            shuffle=self.shuffle
        )
        return test_loader



import torch
from huggingface_hub import hf_hub_download

REPO_ID = "your_username/zuba-language-classifier"
MODEL_FILE = "language_classifier_model2.pth"

def load_model(model_class):
    model_path = hf_hub_download(
        repo_id=REPO_ID,
        filename=MODEL_FILE
    )
    model = model_class(num_labels=4)
    model.load_state_dict(torch.load(model_path, map_location="cpu"))
    model.eval()
    return model



import torch
import tiktoken
from Zuba.Model_training import LanguageClassifier

device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

# Label ID map
id_label = {
    0: "hausa",
    1: "igbo",
    2: "Broken English",
    3: "Yoruba"
}

# Load your model architecture
model = LanguageClassifier(num_labels=4).to(device)

print("Loading model weights...")
model.load_state_dict(torch.load("../model/language_classifier_model2.pth", map_location=device))
model.eval()
print("Model loaded successfully")

class Classify:
    def __init__(self):
        self.tokenizer = tiktoken.get_encoding("gpt2")
        self.max_len = 277
        self.pad_id = 50256

    def encode_text(self, text):
        """Tokenize → truncate → pad"""
        ids = self.tokenizer.encode(text)

        # Truncate
        ids = ids[:self.max_len]

        # Pad
        if len(ids) < self.max_len:
            ids += [self.pad_id] * (self.max_len - len(ids))

        return ids

    def predict(self, text_or_list):
        """Predict label for one text or many."""

        # Handle single vs batch
        if isinstance(text_or_list, str):
            texts = [text_or_list]
            single = True
        else:
            texts = text_or_list
            single = False

        encoded = [self.encode_text(t) for t in texts]

        # Convert to tensor
        input_ids = torch.tensor(encoded, dtype=torch.long).to(device)

        # Model prediction
        with torch.no_grad():
            logits = model(input_ids)

        pred_ids = torch.argmax(logits, dim=1).tolist()
        labels = [id_label[p] for p in pred_ids]

        return labels[0] if single else labels
